/* eslint-disable no-underscore-dangle */
import request from '../utils/request';
import post from '../utils/post';
import config from '../utils/config';
import { Datasource } from './datasource';
import { Interface } from './interfaces';

export async function fetchMenus() {
  return request(`${config.apiContextPath}/api/configure/user/menus`);
}

export async function fetchModuleInfos() {
  return request(`${config.apiContextPath}/api/module/user/info`);
}

export async function fetchModuleLayout(name) {
  return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(name)}/layout`);
}


export const adminApi = {
  async allInfoes(dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/info`, { dmPath });
  },
  async getInfo(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/info`, { dmPath });
  },
  async getConfigure(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/configure`, { dmPath });
  },
  async getPlainConfigure(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/plainConfigure`, { dmPath });
  },
  async setConfigure(name, configure, dmPath) {
    return post(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/configure`, { dmPath, configure });
  },
  async getPlainConfigures(dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/plainConfigures`, { dmPath });
  },
  async setConfigures(configures, dmPath) {
    return post(`${config.apiContextPath}/api/module/admin/configures`, { dmPath, configure: configures });
  },
  async getDocument(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/document`, { dmPath });
  },
  async setDocument(name, document, dmPath) {
    return post(`${config.apiContextPath}/api/module/admin/${encodeURIComponent(name)}/document`, { dmPath, document });
  },
};

export const userApi = {
  async allInfoes(dmPath) {
    return request(`${config.apiContextPath}/api/module/user/info`, { dmPath });
  },
  async getInfo(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(name)}/info`, { dmPath });
  },
  async getLayout(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(name)}/layout`, { dmPath });
  },
  async getDocument(name, dmPath) {
    return request(`${config.apiContextPath}/api/module/user/${encodeURIComponent(name)}/document`, { dmPath });
  },
};

export class Module {
  constructor(name, dmPath) {
    this.name = name;
    this.dmPath = dmPath;
    this._admin = false;
  }
  get admin() {
    return this._admin;
  }
  set admin(admin) {
    this._admin = admin;
  }
  get info() {
    if (this.admin) {
      return adminApi.getInfo(this.name, this.dmPath);
    } else {
      return userApi.getInfo(this.name, this.dmPath);
    }
  }
  get layout() {
    if (this.admin) {
      return adminApi.getLayout(this.name, this.dmPath);
    } else {
      return userApi.getLayout(this.name, this.dmPath);
    }
  }
  get document() {
    if (this.admin) {
      return adminApi.getDocument(this.name, this.dmPath);
    } else {
      return userApi.getDocument(this.name, this.dmPath);
    }
  }
  async setDocument(doc) {
    if (this.admin) {
      return adminApi.setDocument(this.name, doc, this.dmPath);
    }
    throw new Error('Set admin to true before set document.');
  }
  get configure() {
    if (this.admin) {
      return adminApi.getPlainConfigure(this.name, this.dmPath);
    } else {
      return userApi.getPlainConfigure(this.name, this.dmPath);
    }
  }
  async setConfigure(conf) {
    if (this.admin) {
      return adminApi.setConfigure(this.name, conf, this.dmPath);
    }
    throw new Error('Set admin to true before set configure.');
  }
  getDatasource(name) {
    const coordinate = {
      containerType: 'module',
      containerName: this.name,
      datasourceName: name,
    };
    return new Datasource(coordinate, this.dmPath);
  }
  getInterface(name) {
    const coordinate = {
      containerType: 'module',
      containerName: this.name,
      interfaceName: name,
    };
    return new Interface(coordinate, this.dmPath);
  }
}
